#!/bin/sh
#
#        VideoEdge IPSpeedDome Firmware Upgrade
#
#          Copyright 2008 American Dynamics
#

###################### Functions ########################################

fatal()
{
    local result=$?
    echo Error: $* 1>&2
    logger -t $installer -p user.err "Error: $*"
    exit $result
}

warning()
{
    echo Warning: $*
}

# YT 20120606: this should now be copy from one patition to other
extract_backups()
{
    local upgrade_tar=$1
    tar -xzvpOf ${upgrade_tar:?undefined} inner.tar.gz | tar zpvxf - -C /  \
      opt/ad/bin/config_backup.sh \
      opt/ad/bin/camerablocksettings \
      opt/ad/bin/config_encrypt.sh \
      opt/ad/etc/conf/CONFIGBK.conf \
      opt/ad/etc/conf/CONFIGPW.conf || fatal "restore of backup scripts failed"
   echo backup scripts restored
}

# YT20120606 for future use
check_md5()
{
# check md5
	echo "extracting inner tarball md5 checksum"
    tar -C /tmp/ -xpvzf $firmware inner.tar.gz.md5 2>&1 || fatal "unable to extract md5 checksum"

	sample1=`tar -xzOvf $firmware inner.tar.gz|md5sum|head -c 32`
	sample2=`cat /tmp/inner.tar.gz.md5|head -c 32`

	echo "checking md5"
	echo "sample1:$sample1 sample2:$sample2"

   if [ $sample1 != $sample2 ]; then
    	fatal "md5 checksums do not match"
    fi
    echo "md5 checksums match"
}

back_up()
{
############################################################
# Pre-Upgrade phase (delete old files)
############################################################

# convert ANALYTICS.conf if needed
/mnt/flash/opt/ad/bin/analyticssettings

# Extract upgrade versions of backup utilities
# No longer necessary as we run from new partition
# extract_backups $firmware
# Ensure camera block settings are stored correctly
camblkrestorerevision=25 # ipdome.1.0.25.xxx.tar.gz
if [ "$currentrevision" -lt "$camblkrestorerevision" -a "$currentmajor" -eq "1" -a "$currentminor" -eq "0" ]; then
    # Remove old CAMBLK*.conf files if present, keeping only CAMBLK_X22.conf, CAMBLK_X35.conf and CAMBLK_X35_N.conf
    #| egrep -v "CAMBLK_X22\.conf|CAMBLK_X35\.conf|CAMBLK_X35_N\.conf"
    # removed match to these files as they were gen 1 only
    find /opt/ad/etc/conf -name "CAMBLK*.conf" | xargs rm -f
fi

# backup configuration
backup_output=`/mnt/flash/opt/ad/bin/config_backup.sh /`
backup_filename=`echo $backup_output | awk '{print $NF}'`

# Amend bootargs if required
#configure_bootargs

}

mergeImageRotation()
{
    if ! grep "compensated=" /mnt/flash/opt/ad/etc/conf/CAMBLKSETTINGS.conf > /dev/null; then         
        if grep "[ImageRotation]" /mnt/flash/opt/ad/etc/conf/CAMBLKSETTINGS.conf > /dev/null; then         
            sed -i "/\[ImageRotation\]/a\\compensated=False" /mnt/flash/opt/ad/etc/conf/CAMBLKSETTINGS.conf
            sed -i "/compensated=False/a\\pz_compensated=False" /mnt/flash/opt/ad/etc/conf/CAMBLKSETTINGS.conf
            /mnt/flash/usr/bin/sqlite3 /mnt/flash/opt/ad/etc/db/pz.db "                              \
                alter table OSD_PRIVACY_ZONE add column priv_norm_horiz SMALLINT DEFAULT 4096; \
                alter table OSD_PRIVACY_ZONE add column priv_norm_vert SMALLINT DEFAULT 4096;  \
                alter table OSD_PRIVACY_ZONE add column priv_raw_horiz SMALLINT DEFAULT 460;  \
                alter table OSD_PRIVACY_ZONE add column priv_raw_vert SMALLINT DEFAULT 345;  \
            "
        fi                                                                                                           
    fi                               
}

mergeIrCompensateState()
{
    if ! grep "\[IR_COMPENSATE\]" /mnt/flash/opt/ad/etc/conf/PTZFID.conf > /dev/null; then
        echo -e "[IR_COMPENSATE]\nirenabled=1" >> /mnt/flash/opt/ad/etc/conf/PTZFID.conf
    fi                                                                                                          
}

updateConfigFiles()
{
    if pcregrep -M  '\[Exposure\]*(\n)*method=' /mnt/flash/opt/ad/etc/conf/CAMBLKSETTINGS.conf > /dev/null; then
        if ! pcregrep -M 'BrightAreaMode=.*(\n)*Window=' /mnt/flash/opt/ad/etc/conf/CAMBLKSETTINGS.conf > /dev/null; then
            sed -i "/\[Exposure\]/a\\Window=" /mnt/flash/opt/ad/etc/conf/CAMBLKSETTINGS.conf
        fi
    fi
}

restore()
{
    # make a temporary copy of the new PLATMAN.conf
    if [ -f "/mnt/flash/opt/ad/etc/conf/PLATMAN.conf" ];
    then
        cp /mnt/flash/opt/ad/etc/conf/PLATMAN.conf /tmp/PLATMAN.conf
    fi

    # config_restore.sh can accept seperate directories for destination and tools
    /mnt/flash/opt/ad/bin/config_restore.sh $backup_filename /mnt/flash/ /mnt/flash/
    updateConfigFiles
    mergeImageRotation
    mergeIrCompensateState

    # always install the new PLATMAN.conf
    if [ -f "/tmp/PLATMAN.conf" ];
    then
        cp /tmp/PLATMAN.conf /mnt/flash/opt/ad/etc/conf/PLATMAN.conf
    fi
}

merge_files()
{
	# merge conf files
	if [ -f /mnt/flash/opt/ad/bin/config_merge.sh ] ; then
		/mnt/flash/opt/ad/bin/config_merge.sh
	else
		echo "config_merge.sh file does not exist"
	fi
}

# YT: not sure if this belong here or in factory programming.  Place here for now.
initial_setup()
{
	/mnt/flash/opt/ad/bin/config_factory_create.sh
}

############################################################
# Reads the firmware version from the passed MSP430 firmware file
############################################################
get_file_firmware_version()
{
	local FILE=$1
	local DL_ADDR_SIG="S113D000"
	local LSB="00"
	local MSB="00"

	# Get the header line from the file
	line=`grep $DL_ADDR_SIG $FILE`

	# Lens control firmware file has version @ different offsets in line
	if [ "$FILE" == "/lib/firmware/MSP430_LENS.s19" ];
	then
		# Lens Control
		# Get the version low byte
		MSB=`echo $line | cut -c31-32`

		# Get the version hi byte
		LSB=`echo $line | cut -c29-30`
	else
		# PSU & Heater
		# Get the version low byte
		MSB=`echo $line | cut -c23-24`

		# Get the version hi byte
		LSB=`echo $line | cut -c21-22`
	fi
	
	# return the version
	echo "$MSB$LSB"
}



############################################################
# Check for & if present install new MSP430 firmware
############################################################
check_for_msp430_firmware()
{
	local psu=0
	local iob=0
	local lens=0
	local MOUNT_DIR=$1
	local INST_DIR="opt/ad/lib"
	local COPY_DIR="lib/firmware"
	local PSU_FIRMWARE="MSP430_PSU.s19"
	local ENV_FIRMWARE="MSP430_ENV.s19"
	local LENS_FIRMWARE="MSP430_LENS.s19"
	local PSU_DEV_PATH="/sys/bus/i2c/devices/1-0046"
	local ENV_DEV_PATH="/sys/bus/i2c/devices/1-0048"
	local LENS_DEV_PATH="/sys/bus/i2c/devices/2-0040"

	# Make sure the target working directory exists in the new
	# root file system & also in the current working root FS
	# They should always be there but you never know!
	mkdir -p $MOUNT_DIR/$COPY_DIR
	mkdir -p /$COPY_DIR

	# Is there a new PSU firmware file to be installed?
	if [ ! -f "$MOUNT_DIR/$INST_DIR/$PSU_FIRMWARE" ];
	then
		echo "PSU firmware not included in installation package."
	else
		# Copy file from the install directory to the working directory 
		cp $MOUNT_DIR/$INST_DIR/$PSU_FIRMWARE  $MOUNT_DIR/$COPY_DIR/$PSU_FIRMWARE
		if [ $? -ne 0 ];
		then
			echo "Failed to copy $MOUNT_DIR/$INST_DIR/$PSU_FIRMWARE to $MOUNT_DIR/$COPY_DIR/$PSU_FIRMWARE."
			return
		fi
		# Move file from mounted FS to current running root FS working directory
		mv $MOUNT_DIR/$INST_DIR/$PSU_FIRMWARE /$COPY_DIR/$PSU_FIRMWARE
		if [ $? -ne 0 ];
		then
			echo "Failed to move $MOUNT_DIR/$INST_DIR/$PSU_FIRMWARE to /$COPY_DIR/$PSU_FIRMWARE."
			return
		else
			# Set download flag
			psu=1
		fi
	fi

	# Is there a new Env firmware file to be installed?
	if [ ! -f "$MOUNT_DIR/$INST_DIR/$ENV_FIRMWARE" ];
	then
		echo "Env firmware not included in installation package."
	else
		# Copy file from the install directory to the working directory 
		cp $MOUNT_DIR/$INST_DIR/$ENV_FIRMWARE  $MOUNT_DIR/$COPY_DIR/$ENV_FIRMWARE
		if [ $? -ne 0 ];
		then
			echo "Failed to copy $MOUNT_DIR/$INST_DIR/$ENV_FIRMWARE to $MOUNT_DIR/$COPY_DIR/$ENV_FIRMWARE."
			return
		fi
		# Move file from mounted FS to current running root FS working directory
		mv $MOUNT_DIR/$INST_DIR/$ENV_FIRMWARE /$COPY_DIR/$ENV_FIRMWARE
		if [ $? -ne 0 ];
		then
			echo "Failed to move $MOUNT_DIR/$INST_DIR/$ENV_FIRMWARE to /$COPY_DIR/$ENV_FIRMWARE."
			return
		else
			# Set download flag
			iob=1
		fi
	fi

	# Is there a new Lens control firmware file to be installed?
	if [ ! -f "$MOUNT_DIR/$INST_DIR/$LENS_FIRMWARE" ];
	then
		echo "Lens Control firmware not included in installation package."
	else
		# Copy file from the install directory to the working directory 
		cp $MOUNT_DIR/$INST_DIR/$LENS_FIRMWARE  $MOUNT_DIR/$COPY_DIR/$LENS_FIRMWARE
		if [ $? -ne 0 ];
		then
			echo "Failed to copy $MOUNT_DIR/$INST_DIR/$LENS_FIRMWARE to $MOUNT_DIR/$COPY_DIR/$LENS_FIRMWARE."
			return
		fi
		# Move file from mounted FS to current running root FS working directory
		mv $MOUNT_DIR/$INST_DIR/$LENS_FIRMWARE /$COPY_DIR/$LENS_FIRMWARE
		if [ $? -ne 0 ];
		then
			echo "Failed to move $MOUNT_DIR/$INST_DIR/$LENS_FIRMWARE to /$COPY_DIR/$LENS_FIRMWARE."
			return
		else
			# Set download flag
			lens=1
		fi
	fi



	# If there is nothing to download just return
	if [ $psu -eq 0 ] && [ $iob -eq 0 ] && [ $lens -eq 0 ];
	then
		return
	fi

	# Try to download the firmware
	#
	# PSU
	if [ $psu -eq 1 ];
	then
		download_psu_msp430 "/$COPY_DIR/$PSU_FIRMWARE" "$PSU_DEV_PATH"
	fi

	# Lens Control
	if [ $lens -eq 1 ];
	then
		download_lens_control_msp430 "/$COPY_DIR/$LENS_FIRMWARE" "$LENS_DEV_PATH"
	fi

	# Env
	if [ $iob -eq 1 ];
	then
		download_env_msp430 "/$COPY_DIR/$ENV_FIRMWARE" "$ENV_DEV_PATH"
	fi

}

############################################################
# Download firmware to the PSU MSP430
############################################################
download_psu_msp430()
{
	local FIRMWARE=$1
	local DEV_PATH=$2
	local VER_ATTR="$DEV_PATH/msp430"
	local NEW_VER="0000"
	local OLD_VER="0000"
	local MIN_VER="0230"
	local STATUS="XX"
	local DOWNLOAD_ATTR="$DEV_PATH/download"
	local STATUS_ATTR="$DEV_PATH/status"
	local count=0

	
	# Check to see if firmware file exists
	if [ ! -f "$FIRMWARE" ];
	then
		echo "Missing firmware file: $FIRMWARE"
		return
	fi

	
	# Make sure the driver's attributes exists
	if [ ! -e "$STATUS_ATTR" ];
	then
		echo "Missing device driver attributes: $STATUS_ATTR"
		return
	fi

	
	# Get the current firmware version
	OLD_VER=`cat $VER_ATTR`

	# Get the new firmware version
	NEW_VER=`get_file_firmware_version $FIRMWARE`

	# Make sure the current version is upgradable
	if [ "$OLD_VER" -lt "$MIN_VER" ]
	then
		echo "The current PSU firmware can not be upgraded automatically."
		rm -f $FIRMWARE
		return
	fi

	# Make sure the new version is a newer version than what's already running
	if [ "$NEW_VER" -lt "$OLD_VER" ]
	then
		echo "The current PSU firmware(ver:$OLD_VER) is newer than the version in the installation package(ver:$NEW_VER), ignoring."
		rm -f $FIRMWARE
		return
	fi

	# Make sure they are different versions
	if [ "$OLD_VER" == "$NEW_VER" ];
	then
		echo "The new PSU firmware is the same version as the current firmware(ver:$OLD_VER), ignoring."
  		rm -f $FIRMWARE
		return
	fi

	# Start the download, it does NOT block
	echo "Starting PSU firmware download."
	echo 0 > $DOWNLOAD_ATTR
	echo 1 > $DOWNLOAD_ATTR

	# Wait for the download to complete, wait a max of 40 sec
	sleep 2
	STATUS=`cat $DOWNLOAD_ATTR`
	while [ "$STATUS" == "31" ] && [ $count -lt 20 ]
	do
		# Get download status
		STATUS=`cat $DOWNLOAD_ATTR`
		echo "Status = $STATUS"
		let count=$count+1
		sleep 2
	done

	# Check the download result, 30 indicates ok
	STATUS=`cat $DOWNLOAD_ATTR`
	if [ "$STATUS" == "30" ]	 
	then
		# Check for successful restart of the MSP430, wait a max of 20 sec
		STATUS="XX"
		count=0
		while [ "$STATUS" != "ST" ] && [ $count -lt 10 ]
		do
			# Get status
			STATUS=`cat $STATUS_ATTR`
			echo "Status = $STATUS"
			let count=$count+1
			sleep 2
		done
	else
		echo "*** PSU Download failed! ****"
		return 1
	fi

	# Did it restart?
	if [ "$STATUS" != "ST" ];
	then
		echo "MSP430 restart failed."
		return
	fi

	# Get the new firmware version
	NEW_VER=`cat $VER_ATTR`
	echo "Previous version:$OLD_VER. Current version:$NEW_VER"
	echo "PSU download complete."

	# Remove firmware image file
	rm -f $FIRMWARE
}

############################################################
# Download firmware to the Environmental MSP430
############################################################
download_env_msp430()
{
	local FIRMWARE=$1
	local DEV_PATH=$2
	local VERSION_ATTR="$DEV_PATH/msp430"
	local NEW_VER="0000"
	local OLD_VER="0000"
	local MIN_VER="0204"
	local STATUS="XX"
	local DOWNLOAD_ATTR="$DEV_PATH/download"
	local count=0
	local PART_NUM=""

	# Get the camera's assigned part number	
	PART_NUM=$(cat /sys/bus/platform/devices/persistent_data/part_number) &> /dev/null
	if [ $? -ne 0 ];
	then
		echo "Failed to read the camera part number, aborting environmental MSP430 firmware upgrade."
		return
	fi

	# Check length of part number
	if [ ${#PART_NUM} -ne 12 ];
	then
		echo "The part number,[${PART_NUM}] does not seem to be a valid part number, aborting environmental MSP430 firmware upgrade."
		return
	fi

		
	# Check if this camera is supposed to have an environmental/heater MSP340
	if [ "${PART_NUM:7:1}" != "O" ];
	then
		echo "This camera does not have an environmental card, skipping firmware upgrade."
		return
	fi
	
	# Check to see if firmware file exists
	if [ ! -f "$FIRMWARE" ];
	then
		echo "Missing firmware file: $FIRMWARE"
		return
	fi

	
	# Make sure the driver's attributes exists
	if [ ! -e "$DOWNLOAD_ATTR" ];
	then
		echo "Missing device driver attributes: $DOWNLOAD_ATTR"
		return
	fi

	if [ ! -e "$VERSION_ATTR" ];
	then
		echo "Missing device driver attributes: $VERSION_ATTR"
		return
	fi
	
	
	# Get the current firmware version
	OLD_VER=`cat $VERSION_ATTR`

	# Get the new firmware version
	NEW_VER=`get_file_firmware_version $FIRMWARE`
	
	# Make sure the current version is upgradable
	if [ "$OLD_VER" -lt "$MIN_VER" ]
	then
		echo "The current Env firmware can not be upgraded automatically."
		rm -f $FIRMWARE
		return
	fi

	# Make sure the new version is a newer version than what's already running
	if [ "$NEW_VER" -lt "$OLD_VER" ]
	then
		echo "The current Env firmware(ver:$OLD_VER) is newer than the version in the installation package(ver:$NEW_VER), ignoring."
		rm -f $FIRMWARE
		return
	fi

	# Make sure they are different versions
	if [ "$OLD_VER" == "$NEW_VER" ];
	then
		echo "The new Env firmware is the same version as the current firmware(ver:$OLD_VER), ignoring."
	   	rm -f $FIRMWARE
		return
	fi

	# Start the download, it does NOT block
	echo "Starting Env firmware download."
	echo 0 > $DOWNLOAD_ATTR
	echo 1 > $DOWNLOAD_ATTR

	# Wait for the download to complete, wait a max of 70 sec
	sleep 2
	STATUS=`cat $DOWNLOAD_ATTR`
	while [ "$STATUS" == "31" ] && [ $count -lt 35 ]
	do
		# Get download status
		STATUS=`cat $DOWNLOAD_ATTR`
		echo "Status = $STATUS"
		let count=$count+1
		sleep 2
	done

	# Check the download result, 35 indicates ok
	STATUS=`cat $DOWNLOAD_ATTR`
	if [ "$STATUS" != "35" ]	 
	then
		echo "*** Env Download failed! ****"
		return 1
	fi

	# Get the new firmware version
	sleep 3
	NEW_VER=`cat $VERSION_ATTR`
	echo "Previous version:$OLD_VER. Current version:$NEW_VER"
	echo "Env firmware download complete."
	sleep 3

	# Remove firmware image file
	rm -f $FIRMWARE
}


############################################################
# Download firmware to the Lens Control MSP430
############################################################
download_lens_control_msp430()
{
	local FIRMWARE=$1
	local DEV_PATH=$2
	local VERSION_ATTR="$DEV_PATH/msp430"
	local NEW_VER="0000"
	local OLD_VER="0000"
	local MIN_VER="0228"
	local STATUS="XX"
	local DOWNLOAD_ATTR="$DEV_PATH/download"
	local count=0

	
	# Check to see if firmware file exists
	if [ ! -f "$FIRMWARE" ];
	then
		echo "Missing firmware file: $FIRMWARE"
		return
	fi

	
	# Make sure the driver's attributes exists
	if [ ! -e "$DOWNLOAD_ATTR" ];
	then
		echo "Missing device driver attributes: $DOWNLOAD_ATTR"
		return
	fi

	if [ ! -e "$VERSION_ATTR" ];
	then
		echo "Missing device driver attributes: $VERSION_ATTR"
		return
	fi
	
	
	# Get the current firmware version
	OLD_VER=`cat $VERSION_ATTR`

	# Get the new firmware version
	NEW_VER=`get_file_firmware_version $FIRMWARE`
	
	# Make sure the current version is upgradable
	if [ "$OLD_VER" -lt "$MIN_VER" ]
	then
		echo "The current Lens Control firmware can not be upgraded automatically."
		rm -f $FIRMWARE
		return
	fi

	# Make sure the new version is a newer version than what's already running
	if [ "$NEW_VER" -lt "$OLD_VER" ]
	then
		echo "The current Lens Control firmware(ver:$OLD_VER) is newer than the version in the installation package(ver:$NEW_VER), ignoring."
		rm -f $FIRMWARE
		return
	fi

	# Make sure they are different versions
	if [ "$OLD_VER" == "$NEW_VER" ];
	then
		echo "The new Lens Control firmware is the same version as the current firmware(ver:$OLD_VER), ignoring."
	   	rm -f $FIRMWARE
		return
	fi

	# Start the download, it does NOT block
	echo "Starting Lens Control firmware download."
	echo 0 > $DOWNLOAD_ATTR
	echo 1 > $DOWNLOAD_ATTR

	# Wait for the download to complete, wait a max of 80 sec
	sleep 2
	STATUS=`cat $DOWNLOAD_ATTR`
	while [ "$STATUS" == "31" ] && [ $count -lt 40 ]
	do
		# Get download status
		STATUS=`cat $DOWNLOAD_ATTR`
		echo "Status = $STATUS"
		let count=$count+1
		sleep 2
	done

	# Check the download result, 35 indicates ok
	STATUS=`cat $DOWNLOAD_ATTR`
	if [ "$STATUS" != "35" ]	 
	then
		echo "*** Lens Control Download failed! ****"
		return 1
	fi

	# Get the new firmware version
	sleep 20
	NEW_VER=`cat $VERSION_ATTR`
	echo "Previous version:$OLD_VER. Current version:$NEW_VER"
	echo "Lens Control firmware download complete."
	sleep 3

	# Remove firmware image file
	rm -f $FIRMWARE
}



##################### Begin Script Operation ################
exec 200>/tmp/installer.lock
if ! flock -n 200  ; then
   echo "another instance is running";
   logger -t $installer -p user.err "Error: installer already running"
   exit 1
fi

# Direct output from remainder of this script
exec 1>/var/log/update.log 2>&1

echo "Firmware upgrade started at:"
date

if [ 1 -ne $# ]; then
    echo Usage: $0 "<firmware filename>"
    exit 1
fi

firmware=$1
if [ ! -f $firmware ]; then
    fatal "firmware file $firmware doesn't exist"
fi

# Detect which bank is currently active
if [ `fw_printenv activeimage` == "activeimage=A" ] ;
then
  echo Active Image is A
  NEW_KERNEL_MTD=/dev/mtd6
  NEW_FS_MTD=/dev/mtd7
  NEW_MTD_NO=7
  NEW_FS_MTDBLOCK=/dev/mtdblock7
else if [ `fw_printenv activeimage` == "activeimage=B" ] ;
then
  echo Active Image is B
  NEW_KERNEL_MTD=/dev/mtd4
  NEW_FS_MTD=/dev/mtd5
  NEW_MTD_NO=5
  NEW_FS_MTDBLOCK=/dev/mtdblock5
else
  fatal "unable to determine current active image"
fi
fi

# output current firwmare version for information purpose
currentfirmware=`/opt/ad/bin/domeinfo | awk '{print$6}'`
installer=`domeinfo | sed 's/.*Dynamics//' | sed 's/version.*//' | sed 's/\ //g'`_installer.sh

# check if we should install ubifs
ubifs=0

if [ `df -h | grep -c "ubi"` -ne 0 ] ;
then
  ubifs=1
fi

#extract the kernel
#tar zpxf ${firmware} uImage                   || fatal "extracting uImage"
tar zpxf ${firmware} fw.bin --to-stdout | /opt/ad/bin/config_decrypt.sh | tar zpxf - uImage || fatal "extracting uImage"

if [ $ubifs -eq 1 ] ;
then
  # Ensure any previous standby mount is removed.
  umount /mnt/flash &> /dev/null
  ubidetach -d 1 &> /dev/null

  ############### UBI Install #####################
  flash_erase ${NEW_KERNEL_MTD} 0 0                 || fatal "erasing flash partition ${NEW_KERNEL_MTD}"
  nandwrite -p ${NEW_KERNEL_MTD} uImage             || fatal "writing new kernel"

  ubiformat -y ${NEW_FS_MTD} -s 512 -O 2048         || fatal "formatting flash partition ${NEW_FS_MTD}"
  mkdir -p /mnt/flash                               || fatal "creating directory /mnt/flash"
  ubiattach /dev/ubi_ctrl -m ${NEW_MTD_NO} -O 2048  || fatal "ubiattach partition ${NEW_MTD_NO}"

  UBICOUNT=0
  UBISTATUS=0
  UBI1DEV=/dev/ubi1

  echo "Checking ubinfo.."

  # Check that ubiattach has completed before running ubimkvol.
  while [ $UBISTATUS -eq 0 ] && [ $UBICOUNT -lt 10 ]
  do
    ubinfo -d 1

    # the 'ubinfo -d 1' command exits with code 0 if ubi1 device is present.
    if [ $? -eq 0 ] ;
    then
        # also check the ubi1 device file has been created.
        if [ -e $UBI1DEV ] ;
        then
            UBISTATUS=1
        else
            echo "File $UBI1DEV not present"
        fi
    fi
 
    let UBICOUNT=$UBICOUNT+1
    sleep 1
  done

  if [ $UBISTATUS -eq 0 ] ;
  then
    fatal "ubiattach failed to complete"
  fi

  ubimkvol /dev/ubi1 -N rootfs -s 220MiB            || fatal "creating ubi volume"
  mount -t ubifs -o sync ubi1:rootfs /mnt/flash     || fatal "mounting second filesystem"

else

  ############### JFFS2 Install #####################
  flash_erase -j ${NEW_KERNEL_MTD} 0 0          || fatal "erasing flash partition ${NEW_KERNEL_MTD}"
  nandwrite -p ${NEW_KERNEL_MTD} uImage         || fatal "writing new kernel"

  flash_erase -j ${NEW_FS_MTD} 0 0              || fatal "erasing flash partition ${NEW_FS_MTD}"
  mkdir -p /mnt/flash                           || fatal "creating directory /mnt/flash"
  mount -t jffs2 ${NEW_FS_MTDBLOCK} /mnt/flash  || fatal "mounting second filesystem"

fi

############### Explode the new filesystem image #####################
cd /mnt/flash                                 || fatal "Unable to access /mnt/flash"
#tar zpxf ${firmware} fs --strip-components=1  || fatal "writing new filesystem "
tar zpxf ${firmware} fw.bin --to-stdout | /opt/ad/bin/config_decrypt.sh | tar zpxf - fs --strip-components=1 || fatal "writing new filesystem "

############### initialise some device specific items###############
initial_setup

# backup files from configuration list
back_up
# restore the same files
restore

# merge/add new file changes with cooresponding files
merge_files

# create user and group app:apps if not there already
/mnt/flash/opt/ad/bin/set_ug_app-apps.sh /mnt/flash

############### Update MSP430 firmware if needed #####################
check_for_msp430_firmware /mnt/flash

############### Remove factory program indicator if present #########
if [ -e /mnt/flash/opt/ad/bin/factory_program_indicator ] ; then
	rm -f /mnt/flash/opt/ad/bin/factory_program_indicator
fi
############### Remove default lens_type script if persistent_data field already exists ###################
if [ -e /sys/devices/platform/persistent_data/lens_type ]; then
	rm -f /mnt/flash/opt/ad/bin/upgrade_default_lens_type.sh
fi

############### Remove default lens_flange_back_type script if persistent_data field already exists ###################
if [ -e /sys/devices/platform/persistent_data/lens_flange_back_zoom ]; then
	rm -f /mnt/flash/opt/ad/bin/upgrade_default_lens_flange_back.sh
fi

############### Unmount 'new' file system image #####################
cd /tmp
sync
sync
umount /mnt/flash
rmdir /mnt/flash

if [ $ubifs -eq 1 ] ;
then
  ubidetach -d 1
fi

############### Swap active image on next boot & Reboot #####################
fw_setenv swap_image 1                        || fatal "unable to swap images"
echo Rebooting now
reboot

